"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.creepClicks = creepClicks;
exports.creepMousemove = creepMousemove;
exports.creepKeydown = creepKeydown;
exports.creepClipboard = creepClipboard;
exports.creepWheel = creepWheel;
exports.creepSelect = creepSelect;

var _Rx = require("rxjs/Rx");

var _Rx2 = _interopRequireDefault(_Rx);

var _click = require("./events/click");

var _click2 = _interopRequireDefault(_click);

var _doubleclick = require("./events/doubleclick");

var _doubleclick2 = _interopRequireDefault(_doubleclick);

var _multiclick = require("./events/multiclick");

var _multiclick2 = _interopRequireDefault(_multiclick);

var _rightclick = require("./events/rightclick");

var _rightclick2 = _interopRequireDefault(_rightclick);

var _creepmove = require("./events/creepmove");

var _creepmove2 = _interopRequireDefault(_creepmove);

var _shakemove = require("./events/shakemove");

var _shakemove2 = _interopRequireDefault(_shakemove);

var _keydown = require("./events/keydown");

var _keydown2 = _interopRequireDefault(_keydown);

var _cut = require("./events/cut");

var _cut2 = _interopRequireDefault(_cut);

var _copy = require("./events/copy");

var _copy2 = _interopRequireDefault(_copy);

var _paste = require("./events/paste");

var _paste2 = _interopRequireDefault(_paste);

var _wheel = require("./events/wheel");

var _wheel2 = _interopRequireDefault(_wheel);

var _select = require("./events/select");

var _select2 = _interopRequireDefault(_select);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function creepClicks(target, callback) {
  var click$ = _Rx2.default.Observable.fromEvent(target, "click");
  var rightclick$ = _Rx2.default.Observable.fromEvent(target, "contextmenu");

  return _Rx2.default.Observable.merge((0, _click2.default)(click$), (0, _doubleclick2.default)(click$), (0, _multiclick2.default)(click$), (0, _rightclick2.default)(rightclick$)).subscribe(callback);
}

function creepMousemove(target, callback) {
  var mousemove$ = _Rx2.default.Observable.fromEvent(target, "mousemove");

  return _Rx2.default.Observable.merge((0, _creepmove2.default)(mousemove$), (0, _shakemove2.default)(mousemove$)).subscribe(callback);
}

function creepKeydown(target, callback) {
  var keydown$ = _Rx2.default.Observable.fromEvent(target, "keydown");

  return _Rx2.default.Observable.merge((0, _keydown2.default)(keydown$)).subscribe(callback);
}

function creepClipboard(target, callback) {
  var cut$ = _Rx2.default.Observable.fromEvent(target, "cut");
  var copy$ = _Rx2.default.Observable.fromEvent(target, "copy");
  var paste$ = _Rx2.default.Observable.fromEvent(target, "paste");

  return _Rx2.default.Observable.merge((0, _cut2.default)(cut$), (0, _copy2.default)(copy$), (0, _paste2.default)(paste$)).subscribe(callback);
}

function creepWheel(target, callback) {
  var wheel$ = _Rx2.default.Observable.fromEvent(target, "wheel");

  return _Rx2.default.Observable.merge((0, _wheel2.default)(wheel$)).subscribe(callback);
}

function creepSelect(target, callback) {
  var select$ = _Rx2.default.Observable.fromEvent(target, "select");

  return _Rx2.default.Observable.merge((0, _select2.default)(select$)).subscribe(callback);
}

function creep(target, callback) {
  // https://developer.mozilla.org/en-US/docs/Web/Events
  var subClick = creepClicks(target, callback);
  var subMousemove = creepMousemove(target, callback);
  var subKeydown = creepKeydown(target, callback);
  var subClipboard = creepClipboard(target, callback);
  var subWheel = creepWheel(target, callback);
  var subSelect = creepSelect(target, callback);

  var unsubscribe = function unsubscribe() {
    subClick.unsubscribe();
    subMousemove.unsubscribe();
    subKeydown.unsubscribe();
    subClipboard.unsubscribe();
    subWheel.unsubscribe();
    subSelect.unsubscribe();
  };

  return {
    unsubscribe: unsubscribe
  };
}

exports.default = creep;